package com.jiuqi.util.zip;

import java.lang.reflect.InvocationTargetException;
import java.util.NoSuchElementException;
import java.util.zip.ZipException;
import java.lang.reflect.Method;
import java.util.Vector;

public class ZipEntry extends java.util.zip.ZipEntry implements Cloneable
{
    private static final int PLATFORM_UNIX = 3;
    private static final int PLATFORM_FAT = 0;
    private int internalAttributes;
    private int platform;
    private long externalAttributes;
    private Vector extraFields;
    private String name;
    private Long compressedSize;
    private static Method setCompressedSizeMethod;
    private static Object lockReflection;
    private static boolean triedToGetMethod;
    
    static {
        ZipEntry.setCompressedSizeMethod = null;
        ZipEntry.lockReflection = new Object();
        ZipEntry.triedToGetMethod = false;
    }
    
    public ZipEntry(final String name) {
        super(name);
        this.internalAttributes = 0;
        this.platform = 0;
        this.externalAttributes = 0L;
        this.extraFields = new Vector();
        this.name = null;
        this.compressedSize = null;
    }
    
    public ZipEntry(final java.util.zip.ZipEntry entry) throws ZipException {
        super(entry.getName());
        this.internalAttributes = 0;
        this.platform = 0;
        this.externalAttributes = 0L;
        this.extraFields = new Vector();
        this.name = null;
        this.compressedSize = null;
        this.setComment(entry.getComment());
        this.setMethod(entry.getMethod());
        this.setTime(entry.getTime());
        final long size = entry.getSize();
        if (size > 0L) {
            this.setSize(size);
        }
        final long cSize = entry.getCompressedSize();
        if (cSize > 0L) {
            this.setComprSize(cSize);
        }
        final long crc = entry.getCrc();
        if (crc > 0L) {
            this.setCrc(crc);
        }
        final byte[] extra = entry.getExtra();
        if (extra != null) {
            this.setExtraFields(ExtraFieldUtils.parse(extra));
        }
        else {
            this.setExtra();
        }
    }
    
    public ZipEntry(final ZipEntry entry) throws ZipException {
        this((java.util.zip.ZipEntry)entry);
        this.setInternalAttributes(entry.getInternalAttributes());
        this.setExternalAttributes(entry.getExternalAttributes());
        this.setExtraFields(entry.getExtraFields());
    }
    
    protected ZipEntry() {
        super("");
        this.internalAttributes = 0;
        this.platform = 0;
        this.externalAttributes = 0L;
        this.extraFields = new Vector();
        this.name = null;
        this.compressedSize = null;
    }
    
    @Override
    public Object clone() {
        try {
            final ZipEntry e = (ZipEntry)super.clone();
            e.setName(this.getName());
            e.setComment(this.getComment());
            e.setMethod(this.getMethod());
            e.setTime(this.getTime());
            final long size = this.getSize();
            if (size > 0L) {
                e.setSize(size);
            }
            final long cSize = this.getCompressedSize();
            if (cSize > 0L) {
                e.setComprSize(cSize);
            }
            final long crc = this.getCrc();
            if (crc > 0L) {
                e.setCrc(crc);
            }
            e.extraFields = (Vector)this.extraFields.clone();
            e.setInternalAttributes(this.getInternalAttributes());
            e.setExternalAttributes(this.getExternalAttributes());
            e.setExtraFields(this.getExtraFields());
            return e;
        }
        catch (final Throwable t) {
            return null;
        }
    }
    
    public int getInternalAttributes() {
        return this.internalAttributes;
    }
    
    public void setInternalAttributes(final int value) {
        this.internalAttributes = value;
    }
    
    public long getExternalAttributes() {
        return this.externalAttributes;
    }
    
    public void setExternalAttributes(final long value) {
        this.externalAttributes = value;
    }
    
    public void setUnixMode(final int mode) {
        this.setExternalAttributes(mode << 16 | (((mode & 0x80) == 0x0) ? 1 : 0) | (this.isDirectory() ? 16 : 0));
        this.platform = 3;
    }
    
    public int getUnixMode() {
        return (int)(this.getExternalAttributes() >> 16 & 0xFFFFL);
    }
    
    public int getPlatform() {
        return this.platform;
    }
    
    protected void setPlatform(final int platform) {
        this.platform = platform;
    }
    
    public void setExtraFields(final ZipExtraField[] fields) {
        this.extraFields.removeAllElements();
        for (int i = 0; i < fields.length; ++i) {
            this.extraFields.addElement(fields[i]);
        }
        this.setExtra();
    }
    
    public ZipExtraField[] getExtraFields() {
        final ZipExtraField[] result = new ZipExtraField[this.extraFields.size()];
        this.extraFields.copyInto(result);
        return result;
    }
    
    public void addExtraField(final ZipExtraField ze) {
        final ZipShort type = ze.getHeaderId();
        boolean done = false;
        for (int i = 0; !done && i < this.extraFields.size(); ++i) {
            if (this.extraFields.elementAt(i).getHeaderId().equals(type)) {
                this.extraFields.setElementAt(ze, i);
                done = true;
            }
        }
        if (!done) {
            this.extraFields.addElement(ze);
        }
        this.setExtra();
    }
    
    public void removeExtraField(final ZipShort type) {
        boolean done = false;
        for (int i = 0; !done && i < this.extraFields.size(); ++i) {
            if (this.extraFields.elementAt(i).getHeaderId().equals(type)) {
                this.extraFields.removeElementAt(i);
                done = true;
            }
        }
        if (!done) {
            throw new NoSuchElementException();
        }
        this.setExtra();
    }
    
    @Override
    public void setExtra(final byte[] extra) throws RuntimeException {
        try {
            this.setExtraFields(ExtraFieldUtils.parse(extra));
        }
        catch (final Exception e) {
            throw new ZipRuntimeException(e.getMessage());
        }
    }
    
    protected void setExtra() {
        super.setExtra(ExtraFieldUtils.mergeLocalFileDataData(this.getExtraFields()));
    }
    
    public byte[] getLocalFileDataExtra() {
        final byte[] extra = this.getExtra();
        return (extra != null) ? extra : new byte[0];
    }
    
    public byte[] getCentralDirectoryExtra() {
        return ExtraFieldUtils.mergeCentralDirectoryData(this.getExtraFields());
    }
    
    public void setComprSize(final long size) {
        if (haveSetCompressedSize()) {
            performSetCompressedSize(this, size);
        }
        else {
            this.compressedSize = new Long(size);
        }
    }
    
    @Override
    public long getCompressedSize() {
        if (this.compressedSize != null) {
            return this.compressedSize;
        }
        return super.getCompressedSize();
    }
    
    @Override
    public String getName() {
        return (this.name == null) ? super.getName() : this.name;
    }
    
    @Override
    public boolean isDirectory() {
        return this.getName().endsWith("/");
    }
    
    protected void setName(final String name) {
        this.name = name;
    }
    
    private static boolean haveSetCompressedSize() {
        checkSCS();
        return ZipEntry.setCompressedSizeMethod != null;
    }
    
    private static void performSetCompressedSize(final ZipEntry ze, final long size) {
        final Long[] s = { new Long(size) };
        try {
            ZipEntry.setCompressedSizeMethod.invoke(ze, (Object[])s);
        }
        catch (final InvocationTargetException ite) {
            final Throwable nested = ite.getTargetException();
            throw new ZipRuntimeException("Exception setting the compressed size of " + ze + ": " + nested.getMessage());
        }
        catch (final Throwable other) {
            throw new ZipRuntimeException("Exception setting the compressed size of " + ze + ": " + other.getMessage());
        }
    }
    
    private static void checkSCS() {
        if (!ZipEntry.triedToGetMethod) {
            synchronized (ZipEntry.lockReflection) {
                ZipEntry.triedToGetMethod = true;
                try {
                    ZipEntry.setCompressedSizeMethod = java.util.zip.ZipEntry.class.getMethod("setCompressedSize", Long.TYPE);
                }
                catch (final NoSuchMethodException ex) {}
                monitorexit(ZipEntry.lockReflection);
            }
        }
    }
}
